/******************************************************************************
 * Top contributors (to current version):
 *   Gereon Kremer, Aina Niemetz, Morgan Deters
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2024 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Contains code for handling command-line options.
 *
 * For each <module>_options.toml configuration file, mkoptions.py
 * expands this template and generates a <module>_options.h file.
 */

#include "cvc5_private.h"

#ifndef CVC5__OPTIONS__PARSER_H
#define CVC5__OPTIONS__PARSER_H

#include "options/options.h"

// clang-format off

// clang-format on

namespace cvc5::internal::options {

namespace parser::longName {
// clang-format off
  static constexpr const char* filesystemAccess = "filesystem-access";
    static constexpr const char* forceLogicString = "force-logic";
    static constexpr const char* freshBinders = "fresh-binders";
    static constexpr const char* freshDeclarations = "fresh-declarations";
    static constexpr const char* globalDeclarations = "global-declarations";
    static constexpr const char* parseSkolemDefinitions = "parse-skolem-definitions";
    static constexpr const char* semanticChecks = "semantic-checks";
    static constexpr const char* strictParsing = "strict-parsing";
// clang-format on
}

// clang-format off

// clang-format on

#if defined(CVC5_MUZZLED) || defined(CVC5_COMPETITION_MODE)
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT false
#else /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT true
#endif /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */

struct HolderPARSER
{
// clang-format off
  bool filesystemAccess = true;
  bool filesystemAccessWasSetByUser = false;
  std::string forceLogicString = "";
  bool forceLogicStringWasSetByUser = false;
  bool freshBinders = false;
  bool freshBindersWasSetByUser = false;
  bool freshDeclarations = true;
  bool freshDeclarationsWasSetByUser = false;
  bool globalDeclarations = false;
  bool globalDeclarationsWasSetByUser = false;
  bool parseSkolemDefinitions = false;
  bool parseSkolemDefinitionsWasSetByUser = false;
  bool semanticChecks = DO_SEMANTIC_CHECKS_BY_DEFAULT;
  bool semanticChecksWasSetByUser = false;
  bool strictParsing = false;
  bool strictParsingWasSetByUser = false;
// clang-format on
};

#undef DO_SEMANTIC_CHECKS_BY_DEFAULT


}  // namespace cvc5::internal::options

#endif /* CVC5__OPTIONS__PARSER_H */
