/******************************************************************************
 * Top contributors (to current version):
 *   Andrew Reynolds, Hans-Jörg Schurr, Leni Aniva
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2024 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Generated rewrites
 */

#include "expr/node.h"
#include "expr/node_manager.h"
#include "proof/proof_checker.h"
#include "rewriter/rewrite_db.h"
#include "rewriter/rewrites.h"
#include "theory/builtin/generic_op.h"
#include "util/string.h"

using namespace cvc5::internal::kind;

namespace cvc5::internal {
namespace rewriter {

void addRewrites_arith_rewrites(RewriteDb&);
void addRewrites_arrays_rewrites(RewriteDb&);
void addRewrites_booleans_rewrites(RewriteDb&);
void addRewrites_builtin_rewrites(RewriteDb&);
void addRewrites_bv_rewrites(RewriteDb&);
void addRewrites_bv_rewrites_elimination(RewriteDb&);
void addRewrites_bv_rewrites_simplification(RewriteDb&);
void addRewrites_sets_rewrites(RewriteDb&);
void addRewrites_strings_rewrites(RewriteDb&);
void addRewrites_strings_rewrites_regexp_membership(RewriteDb&);
void addRewrites_uf_rewrites(RewriteDb&);
    
    void
    addRules(RewriteDb& db){
        // Calls to individual rewrites
          addRewrites_arith_rewrites(db);
addRewrites_arrays_rewrites(db);
addRewrites_booleans_rewrites(db);
addRewrites_builtin_rewrites(db);
addRewrites_bv_rewrites(db);
addRewrites_bv_rewrites_elimination(db);
addRewrites_bv_rewrites_simplification(db);
addRewrites_sets_rewrites(db);
addRewrites_strings_rewrites(db);
addRewrites_strings_rewrites_regexp_membership(db);
addRewrites_uf_rewrites(db);
            }

Node mkRewriteRuleNode(ProofRewriteRule rule)
{
  return NodeManager::currentNM()->mkConstInt(
      Rational(static_cast<uint32_t>(rule)));
}

bool getRewriteRule(TNode n, ProofRewriteRule& rule)
{
  uint32_t index;
  if (!ProofRuleChecker::getUInt32(n, index))
  {
    return false;
  }
  rule = static_cast<ProofRewriteRule>(index);
  return true;
}

}  // namespace rewriter
}  // namespace cvc5::internal
